(function($, PHD, window, undefined) {
    var $form = $("#form-datetime-appliance"),
        $ntpServerRule = $("#app_ntpserver_checkbox"),
        DateTimeForm,
        DATE_FORMAT = 'mm/dd/yy',
        DATE_DISPLAY_FORMAT = "MM/DD/YYYY",
        TIME_REGEXP = /^([01]?[0-9]|2[0-3])(:[0-5][0-9]){2}$/,
        isrepeat = (typeof(isrepeat) != 'undefined') ? isrepeat : false;

    
    function initForm(){

        $ntpServerRule.on("click", ntpServerUpdate);
        $form.find(".definition").popover({
            appendTo: $form
        });

        DateTimeForm = PHD.FormController($form, {
            serialize: function($form) {
                return $form.find("input").serialize();
            }
        });
        
        DateTimeForm = PHD.FormController($form, {})
        .validate({
            errorLabelContainer: ".datetime-dialog-error-list",
            errorContainer: ".datetime-dialog-errors",
            showErrors: function(errorMap, errorList) {
                $(".datetime-dialog-errors").find(".summary")
                    .html(gettext("Correct the following errors:"));
                this.defaultShowErrors();
            },
            rules: {  
                appliance_date: {
                    required: true
                },
                appliance_time: {
                    required: true,
                    pattern: TIME_REGEXP
                },
                appliance_timezone: {
                    required: true
                }
            },
            messages: {
                appliance_date: {
                        required: gettext("Date is required")
                    },
                appliance_time:{ 
                        required: gettext("Time is required"),
                        pattern: gettext("Time format is not valid")
                     },
                appliance_timezone:{
                        required: gettext("Time Zone is required")
                    } 
            }
        })
        .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
        })
        .on("formsubmit", function(event, data) {
            PHD.currentDialog.wizard("close");
        });
        
        DateTimeForm.submit = function() {
            var self = this
            this.$buttons.blur().disable();

            if ($form.valid()) {
                var dateTimeData = {};
                var ntpServerObj={};

                if($('#app_ntpserver_radioselection').prop("checked") == true){
                    dateTimeData.tz = $('#appliance_timezone option:selected').val();
                    ntpServerObj.enabled   =  true;
                    ntpServerObj.servers = [];
                    var options = document.getElementById("ntpServer").options;
                    if(options != undefined && options != null && options !=""){
                        for(var i=0;i<options.length;i++){
                            if(($('#appliance_ntpserver').val()) != (options[i].value)){
                                ntpServerObj.servers.push(options[i].value);
                            }
                        }
                    }
                    ntpServerObj.servers.push($('#appliance_ntpserver').val());
                    dateTimeData.ntp =  ntpServerObj;
                }else{
                    var dateData = $('#appliance_date').val();
                    var dateDataField = dateData.split("/");
                    dateTimeData.month =  parseInt(dateDataField[0]);
                    dateTimeData.day =  parseInt(dateDataField[1]);
                    dateTimeData.year =  parseInt(dateDataField[2]);

                    var timeData = $('#appliance_time').val();
                    var timeDataField = timeData.split(":");
                    dateTimeData.hour =  parseInt(timeDataField[0]);
                    dateTimeData.minute =  parseInt(timeDataField[1]);
                    dateTimeData.second =  parseInt(timeDataField[2]);
                    
                    dateTimeData.tz = $('#appliance_timezone option:selected').val();
                    ntpServerObj.enabled   = false;
                    dateTimeData.ntp =  ntpServerObj;
                }
            }
            
            dateTimeData = JSON.stringify(dateTimeData);
            
            var url = "/api/date-time/";
            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            url += sid;
            var load = PHD.showLoadingIndicator($form);
            
           PHD.Ajax
                .put(url, dateTimeData, load)
                .done(function(data) {
                    //self.$form.trigger("formsubmit", [data, self.$form, self]);
                })
                .always(function() {
                    self.$buttons.enable();
                });
            
            return this;
        }
        
        $('#edit-appliance').on("beforeactivationtab", function(event,ui) {
            if((ui.oldTab.text()) === " Date Time "){
                var currentData = jQuery.extend(true,{}, ui.currentData);
                var editedDataObject =  jQuery.extend(true,{}, currentData);
                var dateTimeData = {};
                var ntpServerObj={};
                
                var dateData = $('#appliance_date').val();
                var dateDataField = dateData.split("/");
                editedDataObject.month =  parseInt(dateDataField[0]);
                editedDataObject.day =  parseInt(dateDataField[1]);
                editedDataObject.year =  parseInt(dateDataField[2]);

                var timeData = $('#appliance_time').val();
                var timeDataField = timeData.split(":");
                editedDataObject.hour =  parseInt(timeDataField[0]);
                editedDataObject.minute =  parseInt(timeDataField[1]);
                editedDataObject.second =  parseInt(timeDataField[2]);
                
                editedDataObject.tz = $('#appliance_timezone option:selected').val();

                 if($('#app_ntpserver_radioselection').prop("checked") == true){
                     if (editedDataObject.ntp === undefined) {
                         editedDataObject.ntp = {};
                     }
                     editedDataObject.ntp.enabled = true;
                 }else{
                     if (editedDataObject.ntp === undefined) {
                         editedDataObject.ntp = {};
                     }
                     editedDataObject.ntp.enabled = false;
                 }

                var iscompare = compareTwoObjects(currentData,editedDataObject);

                if(iscompare == false){
                   event.preventDefault();
                   PHD.confirmDialog("<p>" + gettext("Do you want to save your changes?") + "</p>", 
                        {title: gettext("Saving Alert")},"Yes")
                    .done(function() {
                       isrepeat = false;
                       $('#edit-appliance').trigger('alertdialogyesclick',isrepeat);
                    })
                    .fail(function() {
                        isrepeat = true;
                        $(currentTarget).trigger('tabsactivateafterconfirm',ui);
                    })
                }else{
                    isrepeat = true;
                    $(currentTarget).trigger('tabsactivateafterconfirm',ui);
                }
            }
        });
    }
    
    $("input[name=datetimeradiogroup]:radio").change(function () {
        if ($("#app_datetime_radioselection").prop("checked")) {
            $('#appliance_ntpserver').attr('disabled','disabled');
            $('#appliance_date').removeAttr('disabled');
            $('#appliance_time').removeAttr('disabled');
            $('#appliance_timezone').removeAttr('disabled');
        }
        else if ($("#app_ntpserver_radioselection").prop("checked")) {
            document.getElementById("appliance_ntpserver").disabled = false;
            $('#appliance_date').attr('disabled','disabled');
            $('#appliance_time').attr('disabled','disabled');
            $('#appliance_timezone').attr('disabled','disabled');
        }
    }); 

    /* Set date picker style */
    $(".datepicker input:text").datepicker({
       showOn: "both",
       buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
       dateFormat: DATE_FORMAT
    }).attr("placeholder", DATE_DISPLAY_FORMAT);

    function ntpServerUpdate(){
        console.log($('#app_ntpserver_checkbox').prop("checked"));
        if($('#app_ntpserver_checkbox').prop("checked") == true){
            $('#appliance_ntpserver').removeAttr('disabled');
            $('#appliance_timezone').attr('disabled','disabled');
            $('#appliance_time').attr('disabled','disabled');
            $('#appliance_date').attr('disabled','disabled');
        }else{
            $('#appliance_ntpserver').attr('disabled','disabled');
            $('#appliance_timezone').removeAttr('disabled');
            $('#appliance_time').removeAttr('disabled');
            $('#appliance_date').removeAttr('disabled');
        }
    }

    function timeZoneValidator(){
        re = /^\d{1,2}:\d{1,2}:\d{1,2}([ap]m)?$/; 
        var timeFieldVal = document.getElementById("appliance_time").value;
         /* To check root password and Confirm matching or not. */
            $.validator.addMethod("isTimeZoneMatch", function(value, element) { 
                      if(timeFieldVal.match(re))
                            return true;
                        else
                            return false;

            }, "Invalid time format");
    }
    
     /* DOM Ready Function */
    $(function() {
        initForm();
    });
    
})(jQuery, window.PHD || (window.PHD = {}), window);

