angular.module('configure_server',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('ConfigureServerCtrl', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http, ngDialog) {

        var selectedCredential;
        var associatedAppliance;
        var serverEdit = false;
        var selectedRetention;
        var assetId;

        $scope.initCtrl = function(dlg){
            switch (dlg.name){
                case 'Add-Server':
                    console.log("Adding Asset...");
                    break;
                case 'Edit-Server':
                    serverEdit = true;
                    $scope.editProcessing($scope.ngDialogData);
                    break;
                case 'Edit-Assets':
                    serverEdit = true;
                    $scope.editAssetProcessing($scope.ngDialogData);
                    break;
            }
            processDialog(PHD, window);
        };

        $scope.editProcessing = function(asset){
            assetId = asset.id;
            $("#retentionSection").show();
            $("#id_installAgentDiv").hide();
            $("#add-edit-asset-dialog-title").text("Edit Asset");
            $("#id_server").val(asset.name);
            $("#id_ip").val(asset.ip);
            $("#edit_object_id").text(asset.id);
            $("#id_enabled").prop('checked', asset.is_enabled);
            $("#id_encrypted").prop('checked', asset.is_encrypted);
            $('#id_encrypted').prop('disabled', true);
            selectedCredential = asset.credentials != undefined ? asset.credentials.credential_id : -1;
            selectedRetention = asset.retention != undefined ? asset.retention.id_retentionPolicy : 0;
            associatedAppliance = asset.system_id;
            $scope.getApplianceEncryptionStatus(associatedAppliance, asset.is_encrypted);
        };

        $scope.getApplianceEncryptionStatus = function(associatedAppliance, isAssetEncrypted){
            var resp = PHD.Ajax.get("/api/encryption/?sid=" + associatedAppliance);
            resp.done(function (data) {
                data = data.data[0];
                /* Enable Encrypt backup checkbox, for following scenarios
                   1) Edit Asset, Appliance Encryption ON/PERSIST
                   2) Edit Asset, Appliance Encryption OFF and Asset Encryption ON
                   3) Add Asset, Appliance Encryption ON/PERSIST
                */
                if ( (data.state != "off" && serverEdit) || (data.state == "off" && serverEdit && isAssetEncrypted) || (!serverEdit && data.state != "off") ) {
                    $('#id_encrypted').prop('disabled', false);
                    $('#id_encrypted').prop('title', "");
                }
                else if (!serverEdit && data.state == "off") {
                    $('#id_encrypted').prop('disabled', true);
                    $('#id_encrypted').prop('title', "Backups cannot be encrypted as the associated appliance does not support encryption.");
                    $("#id_encrypted").prop('checked', isAssetEncrypted);
                }
            })
        };

        $scope.editAssetProcessing = function(assets){
            $scope.assets = assets;
            $("#retentionSection").show();
            $("#add-edit-asset-dialog-title").text("Edit Asset");
            $("#id_server").hide();
            selectedCredential = -1;
            selectedRetention =  0;
            associatedAppliance = null;
        };

        function handleAssetError(jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);
                    PHD.hideLoadingIndicator(AddAssetIndicator);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'ErrorDialog',
                        scope:$scope,
                        dialogMessage: error,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }


        function handlePolicyError(jqXHR, textStatus, errorThrown) {
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined &&
                    data.result[0].message.indexOf("strategy of the system is not GFS") != -1) {
                    console.log("Not GFS, don't allow assignment of policies");
                    $scope.isPolicyGFS = false;
                    $('#retentionSection').prop('disabled', true);
                    $('.retentionPolicyList').prop('disabled', true);
                    $('.retentionPolicyList').attr('title', data.result[0].message);
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.launchCredentialManager = function() {
            ngDialog.open({
                template: 'app/configure/credentials/credentials.html',
                scope: $scope,
                overlay:true,
                name: 'Credential-Manager',
                ngDialogStyle:'width:700px; height:550px;',
                closeByDocument: false,
                closeByEscape: false,
                preCloseCallback: function (value) {
                    processDialog(PHD,window);              //Update the dialog contents after returning from "Manage Credentials"
                }
            });
        };

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        function processDialog(PHD, window) {
            $form = $("#config-server-dialog");

            $("#retentionSection").hide();
            var credentials = [];
            var credentialsSelect;
            var $credentialsSelect = $("#credential_id");
            var applianceSelect;
            var $applianceSelect = $("#id_system");
            var $installAgent = $("#id_installAgent");
            var retentionPolicySelect;
            var selectedCredentialID = 0;

            function sortIDs(a, b) {
                return (a.id < b.id) ? -1 : ((a.id > b.id) ? 1 : 0);
            }

            applianceSelect = $applianceSelect[0];
            applianceSelect.options.length = 0;

            function refreshSystemList() {
                PHD.appliance_sid = 1; // default to local
                console.log('set appliance id to ' + PHD.appliance_sid);
                var systems = PHD.Ajax.get('/api/systems');
                systems.done(function(data) {
                    var appliances = data.appliance.sort(sortIDs);
                    appliances.forEach(function (system, index) {
                        applianceSelect.options[index] = new Option(system.name, system.id);
                    });
                    if (serverEdit) {
                        $("#id_system").val(parseInt(associatedAppliance));
                        $("#id_system").prop("disabled", true);
                        $("#host_name_tooltip").text("Hostname cannot be changed.");
                    }
                });
            }

            function refreshRetentionList() {
                var retention = PHD.Ajax.get('/api/retention/policy/', null, handlePolicyError);
                retention.done(function(data){
                    $scope.isPolicyGFS = true;
                    data.data.forEach(function (system, index) {
                        retentionPolicySelect.options[index] = new Option(system.name, system.id);
                    });
                });
            }

            function populateCredentials() {
                credentialsSelect = $credentialsSelect[0];
                credentialsSelect.options.length = 0;
                var i = 0;
                if(serverEdit){
                    credentialsSelect.options[i] = new Option('None', -1);
                    i++;
                }
                credentialsSelect.options[i] = new Option('New', 0);
                if (credentials.length > 0) {
                    credentials.forEach(function (credential, index) {
                        if (credential.display !== null && credential.display !== undefined && credential.display.length > 0) {
                            credentialsSelect.options[index + (i+1)] = new Option(credential.display, credential.credential_id);
                        }
                    })
                }
            }

            function initForm() {

                $form.find(".definition").popover({
                    appendTo: $form
                });

                var systemsList = PHD.Ajax.get('/api/systems');
                systemsList.done(function (data) {
                    var appliances = data.appliance.sort(sortIDs);
                    appliances.forEach(function(system, index) {
                        applianceSelect.options[index] = new Option(system.name, system.id);
                    });
                });

                $applianceSelect.on("change", function (event) {
                    $scope.getApplianceEncryptionStatus($applianceSelect.val(), false);
                });

                var resp = PHD.Ajax.get('/api/credentials');
                resp.done(function (data) {
                    console.log("GET credentials");
                    credentials = data.data;
                    credentials.forEach(function (credential, index) {
                        if(credential.display_name != undefined){
                            var display = credential.display_name;
                            if (credential.is_default) {
                                display += " (Default)";
                            }
                            credential.display = display;
                        }
                        else{
                            credentials.splice(credentials.indexOf(credential),1);
                        }
                    });
                    populateCredentials();

                    $installAgent.on("change", function (event) {
                        if($("#id_installAgent").prop("checked")) {
                            $("#id_CredentialsDiv").show();
                            $(".modal-form-credential").show();
                            $("#credential_id").val(0);
                            $('#id_btnSaveAsset').prop('disabled', true);
                            $scope.setDialogHeight(645);
                            $(".ngdialog-titlebar").click(); // Added to workaround the dialog height adjustment problem in FireFox
                        }
                        else{
                            $("#id_CredentialsDiv").hide();
                            $('#id_btnSaveAsset').prop('disabled', false);
                            $scope.setDialogHeight(360);
                            $(".ngdialog-titlebar").click();// Added to workaround the dialog height adjustment problem in FireFox
                        }
                    });


                    $scope.setDialogHeight = function(height){
                        $("#resizableDialog").one( "click", function() {
                            $(".ngdialog-content").height(height).css({
                                height: height + "px;"
                            });
                        });
                    };

                    $(".modal-form-credential").hide();
                    $credentialsSelect.on("change", function (event) {
                        var dialogHeight;
                        selectedCredentialID = $credentialsSelect.val();
                        if (selectedCredentialID == 0) {
                            dialogHeight = serverEdit ? 698 : 665;
                            $(".modal-form-credential" ).show();
                            $("#id_credentials_displayName").val("");
                            $("#id_credentials_username").val("");
                            $("#id_credentials_password").val("");
                            $('#id_btnSaveAsset').prop('disabled', true);
                        } else {
                            $( ".modal-form-credential" ).hide();
                            dialogHeight = serverEdit ? 520 : 470;
                            $('#id_btnSaveAsset').prop('disabled', false);
                        }
                        $scope.setDialogHeight(dialogHeight);
                    });

                    if(serverEdit){
                        $("#credential_id").val(parseInt(selectedCredential));
                        $('#retentionSection').show();
                        $("#id_system").val(parseInt(associatedAppliance));
                        $("#id_retentionPolicy").val(parseInt(selectedRetention));
                        $("#id_server").prop("disabled", true);
                        $('#id_btnSaveAsset').prop('disabled', false);
                    }
                    else{
                        $('#id_btnSaveAsset').prop('disabled', true);
                        $("#id_CredentialsDiv").show();
                        $(".modal-form-credential").show();
                        $("#credential_id").val(0);
                        if ($rootScope.isCE) {
                            $installAgent.prop('checked', false);
                            $installAgent.trigger('change');
                            $installAgent.disable();
                        }
                    }
                });
                refreshSystemList();
                if (serverEdit) {
                    $('#retentionSection').show();
                    retentionPolicySelect = $("#id_retentionPolicy");
                    retentionPolicySelect = retentionPolicySelect[0];
                    refreshRetentionList();
                }
                else
                    $scope.getApplianceEncryptionStatus(PHD.appliance_sid, false);
            }
            initForm();

            $scope.AddAsset = function () {
                console.log("Save asset clicked - serverEdit : "+ serverEdit);
                var obj={};
                // associatedAppliance is being passed as query param. Form element(id_system) is not used by the API.
                var associatedAppliance = $("#id_system").val();
                var installAgent = $("#id_installAgent").prop("checked") ? true : false;
                obj.name = $("#id_server").val();
                obj.os_type = "Windows";
                obj.priority = 300;
                obj.is_enabled = true;
                obj.is_synchable = false;
                obj.use_ssl = false;
                obj.install_agent = installAgent;
                obj.is_auth_enabled = (selectedCredentialID == -1) ? false : true ;
                obj.is_encrypted = $("#id_encrypted").prop("checked") ? true : false;

                var hostinfo = {};
                hostinfo.ip = $("#id_ip").val();

                var credentials = {};

                console.log("selected = " + selectedCredentialID);
                if (installAgent) {
                    if (selectedCredentialID == 0) {
                        credentials.display_name = $("#id_credentials_displayName").val();
                        credentials.username = $("#id_credentials_username").val();
                        credentials.password = $("#id_credentials_password").val();
                        credentials.is_default = $("#id_default").prop("checked");
                        obj.credentials = credentials;
                    }
                    else if (selectedCredentialID == -1) {
                        obj.credential_id = -1;
                    }
                    else {
                        obj.credential_id = parseInt(selectedCredentialID);
                    }
                }
                obj.host_info = hostinfo;

                var base_url = "/api/clients/";

                obj = JSON.stringify(obj);
                if(serverEdit){
                    console.log("selected retentionPolicy = " + $("#id_retentionPolicy").val());
                    //TODO: VALIDATE the retentionID usage once the API is ready to attach the retention policy with the Asset.
                    obj.retention_id = $("#id_retentionPolicy").val();
                    console.log("PUT called to edit Asset with ID : " + assetId + " -- associatedAppliance : " + associatedAppliance);
                    base_url +=assetId + "/";
                    AddAssetIndicator = PHD.showLoadingIndicator("body", true, "Editing ...");
                    resp = PHD.Ajax.put(base_url + "?sid=" +  associatedAppliance, obj, null, handleAssetError);
                }
                else{
                    console.log("POST called to add an Asset.associatedAppliance .  "+associatedAppliance);
                    AddAssetIndicator = PHD.showLoadingIndicator("body", true, "Adding Asset...");
                    resp = PHD.Ajax.post(base_url + "?sid=" +  associatedAppliance, obj, null, handleAssetError);
                }
                resp.done(function(data) {
                    PHD.hideLoadingIndicator(AddAssetIndicator);
                    processDialog(PHD, window);
                    $scope.closeThisDialog();
                    var result = data.result;
                    if (Array.isArray(result)) {
                        result = result[0];
                    }
                    if (result !== undefined) {
                        if (parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                            ngDialog.close("Info-Error-Dialog");
                            console.log("SUCCESS");
                            $(document).trigger("assetChange");
                        } else if (parseInt(result.code) === AJAX_RESULT_ERROR) {
                            console.log("FAILURE");
                            ngDialog.open({
                                dialogType: 'ERROR',
                                modelDialogId:'Info-Error-Dialog',
                                dialogMessage: data,
                                overlay: true
                            });
                        } else {
                            // adding server, get back id.
                            console.log("SUCCESS");
                            $(document).trigger("assetChange");
                        }
                    }

                });

            };

            $scope.editMultipleAssets = function () {
                console.log("Save asset clicked - serverEdit : "+ serverEdit);

                var obj = {};
                var credentials = {};

                console.log("selected = " + selectedCredentialID);
                if(selectedCredentialID == 0){
                    credentials.display_name = $("#id_credentials_displayName").val();
                    credentials.username = $("#id_credentials_username").val();
                    credentials.password = $("#id_credentials_password").val();
                    credentials.is_default = $("#id_default").prop("checked");
                    obj.credentials = credentials;
                }
                else if(selectedCredentialID == -1){
                    obj.credential_id = -1;
                }
                else{
                    obj.credential_id = parseInt(selectedCredentialID);
                }

                var client_url = "/api/credentials/bind/client/";
                var instance_url = "/api/credentials/bind/instance/";

                var putObj = JSON.stringify(obj);

                var hasErrors = false;

                $scope.assets.forEach(function(asset) {
                    var resp;
                    if (asset.asset_type === ASSET_TYPE_PHYSICAL) {
                        obj.clientID = asset.id;
                        putObj = JSON.stringify(obj);
                        resp = PHD.Ajax.put(client_url + obj.credential_id + "/?sid=" + asset.system_id, putObj, null, null);
                    } else {
                        obj = {};
                        obj.credential_id = parseInt(selectedCredentialID);
                        obj.app_aware = true; // TODO, add to dialog for a VM.
                        putObj = JSON.stringify(obj);
                        resp = PHD.Ajax.put(instance_url + asset.id + "/?sid=" + asset.system_id, putObj, null, null);
                    }

                    resp.done(function(data) {
                        if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                            var error = data.result[0].message;
                            console.log("The error is:" + error);
                            if (error != undefined) {
                                hasErrors = true;
                            }
                        }
                    });
                });

                if (!hasErrors) {
                    $scope.closeThisDialog();
                    $(document).trigger("assetChange");
                }

            };
        }

       // Initialize the controller.
          $scope.$on('ngDialog.opened', function(event,obj) { $scope.initCtrl(obj) });
    }]);
