angular.module('iscsiStorage', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('iscsiStorageController', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http,ngDialog) {

        var StorageEdit = false;
        var AddStorage,isStatelessData;
        var $targetSelect = $("#id_target_list");
        var $lunSelect = $("#id_lun_list");

        $scope.initCtrl = function(obj){

            $("#stateless_requirement_text").hide();
            switch(obj.name){
                case 'iSCSI-backup':
                    $scope.iscsi = {};
                    $scope.iscsi.target = '';
                    $scope.type = "backup";
                    isStatelessData = $scope.ngDialogData;
                    if(isStatelessData.isStateless){
                        $scope.iscsi.name = "Internal";
                        $("#id-name").disable();
                        $scope.type = "stateless";
                        $scope.build_filesystem = 1;
                        $("#stateless_requirement_text").show();
                    }
                    break;
                case 'iSCSI-archive':
                    $scope.iscsi = {};
                    $scope.iscsi.target = '';
                    $scope.type = "archive";
                    $("#title").text('Add iSCSI Backup Copy Target');
                    break;
                case 'iSCSI-Edit-Backup':
                    $scope.iscsi = {};
                    iscsiData = $scope.ngDialogData;
                    $scope.type = "backup";
                    $scope.iscsi.name = iscsiData.name;
                    $("#title").text('Edit iSCSI Storage');
                    $("#id-name").disable();
                    $scope.iscsi.host = iscsiData.properties.hostname;
                    $scope.iscsi.port = iscsiData.properties.port;
                    $scope.iscsi.target = iscsiData.properties.target;
                    $scope.iscsi.lun = iscsiData.properties.lun;
                    $scope.id = iscsiData.id;
                    StorageEdit = true;
                    break;
                case 'iSCSI-Edit-Archive':
                    $scope.iscsi = {};
                    iscsiData = $scope.ngDialogData;
                    $scope.type = "archive";
                    $scope.iscsi.name = iscsiData.name;
                    $("#title").text('Edit iSCSI Backup Copy Target');
                    $("#id-name").disable();
                    $scope.iscsi.host = iscsiData.properties.hostname;
                    $scope.iscsi.port = iscsiData.properties.port;
                    $scope.iscsi.target = iscsiData.properties.target;
                    $scope.iscsi.lun = iscsiData.properties.lun;
                    $scope.id = iscsiData.id;
                    StorageEdit = true;
                    break;
            }
            processDialog(PHD, window);
        };

        function processDialog(PHD, window){
            var $port = $("#id_port"),
                portSpinnerOptions = {"min": 1, "max": 10000, "step": 1};

            $port.spinner(portSpinnerOptions);
            $scope.iscsi.port = 3260;
        }

        $scope.submitForm = function () {
            var obj = {};
            var properties = {};
            if(!StorageEdit){
                obj.name = $scope.iscsi.name;
                obj.usage = $scope.type;
                properties.username = $scope.iscsi.username;
                properties.password = $scope.iscsi.password;
                if($scope.type === "stateless"){
                    obj.name = "_Stateless";
                    obj.build_filesystem = 1;
                }
            }else{
                console.log("Editing");
            }

            obj.type = 1;
            properties.port = $scope.iscsi.port;
            properties.hostname = $scope.iscsi.host;
            properties.target = $scope.iscsi.target;
            properties.lun = parseInt($scope.iscsi.lun);
            obj.properties = properties;

            console.log(obj);

            var base_url = "/api/storage/";

            obj = JSON.stringify(obj);
            if(StorageEdit){
                console.log("PUT called");
                base_url +=$scope.id + "/";
                AddStorage = PHD.showLoadingIndicator("body", true, "Saving...");
                resp = PHD.Ajax.put(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
            }
            else{
                if($scope.type == "archive") {
                    AddStorage = PHD.showLoadingIndicator("body", true, "Adding and checking for existing Backup Copies.  Please wait...");
                } else {
                    AddStorage = PHD.showLoadingIndicator("body", true, "Adding Storage...");
                }
                resp = PHD.Ajax.post(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
            }

            resp.done(function(data) {

                PHD.hideLoadingIndicator(AddStorage);

                if($scope.type == "archive") {
                    var result = data.storage;
                    console.log("SUCCESS");
                    var dialogType = "Information";
                    if(result['messages'] != undefined && Array.isArray(result['messages'])) {
                        dialogType = "Warning";
                        var message = '';
                        for(var i=0; i<result['messages'].length;i++){
                            message = message +"<br>"+result.messages[i];
                        }
                    } else if (result['sets_needing_import'] == 0) {
                        message = 'No backup copies found needing import';
                    } else {
                        message = 'All backup copies imported successfully';
                    }

                    ngDialog.open({
                        dialogType:dialogType,
                        dialogMessage:message,
                        modelDialogId:'InfoDialog',
                        scope:$scope,
                        onConfirmOkButtonClick:'onConfirmInfo()'
                    });
                } else {
                    var result = data.result[0];
                    if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                        console.log("SUCCESS");
                        processDialog(PHD, window);
                        $scope.closeThisDialog();
                    } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                        console.log("FAILURE");
                        ngDialog.open({
                            dialogType: 'ERROR',
                            dialogMessage: data,
                            overlay:true
                        });
                    }
                }

                //PHD.currentDialog.wizard("close");
            });

        };

        $scope.TargetChange = function() {
            console.log("Changes");
            var selectedtarget = $targetSelect.val();
            console.log(selectedtarget);
            $scope.iscsi.target = selectedtarget;
        };

        $scope.ScanTargets = function() {
            $targetSelect = $("#id_target_list");
            var scan_url = "/api/storage/targets/";
            ScanIndicator = PHD.showLoadingIndicator("body", true, "Searching for Targets...");
            resp = PHD.Ajax.get(scan_url + "?sid=" +  PHD.appliance_sid + "&host=" + $scope.iscsi.host + "&port=" + $scope.iscsi.port, null, handleStorageError);

            $("#backup-job-form-content").width(800);
            resp.done(function(data) {
                PHD.hideLoadingIndicator(ScanIndicator);
                var targets = data.targets;
                if(data.targets != null){
                    targetSelect = $targetSelect[0];
                    targetSelect.options.length = 0;
                    if (targets.length > 0) {
                        targets.forEach(function (target, index) {
                            targetSelect.options[index] = new Option(target);
                        })
                    }
                }
                else{
                    ngDialog.open({
                        dialogType:'Warning',
                        modelDialogId:'SelectCredential',
                        dialogMessage: 'No targets were found.'
                    });
                }
            });
        };

        $scope.LUNChange = function() {
            console.log("Changes");
            var selectedLUN = $lunSelect.val();
            console.log(selectedLUN);
            $scope.iscsi.lun = selectedLUN;
        };

        $scope.ScanLUNs = function() {
            $lunSelect = $("#id_lun_list");
            var scan_url = "/api/storage/targets/";
            var ScanIndicator = PHD.showLoadingIndicator("body", true, "Searching for LUNs...");
            resp = PHD.Ajax.get(scan_url + $scope.iscsi.target +"/?sid=" +  PHD.appliance_sid + "&host="+ $scope.iscsi.host + "&port="+ $scope.iscsi.port, null, handleStorageError);

            resp.done(function(data) {
                PHD.hideLoadingIndicator(ScanIndicator);
                var luns = data.luns;
                if(data.luns != null){
                    lunSelect = $lunSelect[0];
                    lunSelect.options.length = 0;
                    if (luns.length > 0) {
                        luns.forEach(function (lun, index) {
                            lunSelect.options[index] = new Option(lun);
                        })
                    }
                }
                else{
                    ngDialog.open({
                        dialogType:'Warning',
                        modelDialogId:'SelectCredential',
                        dialogMessage: 'No targets were found.'
                    });
                }
            });
        };

        function handleStorageError(jqXHR, textStatus, errorThrown) {
            PHD.hideLoadingIndicator(AddStorage);
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);

                    if(error.indexOf("Storage configuration failed - this volume contains existing data, are you sure you want to format it?") > -1){               //Check if storage contains existing data, so we can show user option to recover.
                        var modifiedMessage = "The selected storage contains an existing filesystem.  If it contains Unitrends backup data, you can attempt to recover it.  Otherwise, you can erase the data or cancel and select different storage."
                        ngDialog.open({
                            dialogType:'CONFIRMATION - STORAGE',
                            modelDialogId:'ChooseRecoverOrErase',
                            scope:$scope,
                            dialogMessage: modifiedMessage,
                            tertiaryButton: true,
                            tertiaryButtonLabel: "Erase Data",
                            onConfirmOkButtonClick:'onRecover()',
                            onTertiaryButtonClick:'OnErase()'
                        });
                    }
                    else{
                        ngDialog.open({
                            dialogType:'Error',
                            modelDialogId:'ErrorDialog',
                            scope:$scope,
                            dialogMessage: error,
                            onConfirmOkButtonClick:'onConfirmError()'
                        });
                    }
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        $scope.onConfirmInfo = function () {
            ngDialog.close('InfoDialog');
            processDialog(PHD, window);
            $scope.closeThisDialog();
        };

        $scope.onRecover = function () {
            console.log("Recover the storage");
            $scope.build_filesystem = 0;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.OnErase = function () {
            console.log("Erase the Data");
            $scope.build_filesystem = 2;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });
    }]);

/*(function($, PHD, window, undefined) {
    
    var $form = $("#bds-form"),
        CIFSForm,
        $port = $("#id_port"),
        $d2d = $("#id_d2d_allocation"),
        $ir = $("#id_ir_allocation"),
        CIFS_REGEX = /^\\\\[a-zA-Z0-9\-\_\.]+\\.+$/,
        NAME_REGEX = /^[a-zA-Z0-9\-\_]+$/,
        $storageID = $("#edit_object_id"),
        $systemID = $("#id_system"),
        portSpinnerOptions = {"min": 1, "max": 10000, "step": 1},
        $toggleCtrl = $form.find(".formset-toggle :checkbox"),
        system_id = PHD.appliance_sid,
    allocationSpinnerOptions = {"min": 1, "max": 99, "step": 1};

    function initForm() {
        $form.find(".definition").popover({
            appendTo: $form,
            position: {
                my: "left bottom",
                at: "right top-15"
            }
        });

        $port.spinner(portSpinnerOptions);
        $d2d.spinner(allocationSpinnerOptions);
        $ir.spinner(allocationSpinnerOptions);

        CIFSForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    name: {
                        required: true,
                        pattern: NAME_REGEX
                    },
                    host: {
                        required: true
                    },
                    share: {
                        required: true
                    }
                    /*
                    path: {
                        required: true,
                        pattern: CIFS_REGEX
                    },
                    */
            /*    },
                messages: {
                    name: {
                        required: gettext("Please name your Storage")
                    },
                    host: {
                        required: gettext("Please provide the CIFS hostname or IP address")
                    },
                    share: {
                        required: gettext("Please provide the CIFS share name")
                    }
                    /*
                    path: {
                        required: gettext("Please enter the CIFS path"),
                        pattern: gettext("Please enter a valid CIFS path")
                    },
                    */
            /*    }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                $(document).trigger("bdssave", [system_id, data]);
                PHD.currentDialog.wizard("close");
            });
            /*
            .on("formsubmit", function(event, data) {
                    PHD.confirmDialog('<p>' + data.result[0].message + '</p>', {'title': gettext('Confirm Storage Edit')})
                        .done(function() {
                            $override.appendTo($form);
                            BDSForm.submit();
                        });
                }
            });
            */

     /*   if($toggleCtrl.length) {
            var $toManage = $("#id_password, #id_username").prop("disabled", true);

            $toggleCtrl.on("change", function(event) {
                $toManage.prop("disabled", !$(this).prop("checked"));
            });
        }
    }

    /*
    *   DOM Ready Function
    */
   /* $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
        });

    });

})(jQuery, window.PHD || (window.PHD = {}), window); */
