angular.module('nfsStorage', ['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('NfsStorageController', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http,ngDialog) {

        var StorageEdit = false;
        var AddStorage, isStatelessData;
        var nfsData = {};

        $scope.initCtrl = function(obj){

            $scope.sizes = [{code: 1, metric: 'GB'}, {code: 2, metric: 'TB'}];
            $("#stateless_requirement_text").hide();
            switch(obj.name){
                case 'NFS-backup':
                    $scope.nfs = {};
                    $scope.type = "backup";
                    $scope.nfs.percent = 100;
                    $scope.pattern = /^([a-zA-Z0-9_-]){0,9}$/;
                    $scope.nameErrorString = "Please use shorter name with no space";
                    $("#storage-allocation").hide();
                    isStatelessData = $scope.ngDialogData;
                    if(isStatelessData.isStateless){
                        $scope.nfs.name = "Internal";
                        $("#id-name").disable();
                        $scope.type = "stateless";
                        $scope.build_filesystem = 1;
                        $("#stateless_requirement_text").show();
                    }
                    break;
                case 'NFS-archive':
                    $scope.nfs = {};
                    $scope.type = "archive";
                    $("#title").text('Add NFS Backup Copy Target');
                    $("#storage-allocation").hide();
                    $scope.pattern = /^[a-zA-Z0-9_-]*$/;
                    $scope.nameErrorString = "Name should not have any space";
                    break;
                case 'NFS-Edit-Backup':
                    $scope.type = "backup";
                    $scope.nfs = {};
                    $scope.pattern = /^([a-zA-Z0-9_-]){0,9}$/;
                    $scope.nameErrorString = "Please use shorter name with no space";
                    StorageEdit = true;
                    nfsData = $scope.ngDialogData;
                    $scope.nfs.name = nfsData.name;
                    $("#title").text('Edit NFS storage');
                    $("#id-name").disable();
                    $scope.nfs.host = nfsData.properties.hostname;
                    $scope.nfs.port = nfsData.properties.port;
                    $scope.nfs.share = nfsData.properties.share_name;
                    $scope.nfs.username = nfsData.properties.username;
                    $scope.nfs.password = nfsData.properties.password;
                    $scope.nfs.mb_available = nfsData.properties.mb_available;
                    gb_available = $scope.nfs.mb_available/1024;
                    $scope.nfs.max_size = nfsData.properties.max_size;
                    $scope.nfs.percent = Math.round(($scope.nfs.max_size/gb_available)*100);
                    $scope.nfs.sizeValue = nfsData.properties.max_size;
                    $scope.nfs.size = 1;
                    if( $scope.nfs.sizeValue > 1024){
                        $scope.nfs.sizeValue = $scope.nfs.sizeValue / 1024;
                        $scope.nfs.size = 2;
                    }
                    $scope.id = nfsData.id;
                    if(nfsData.type == "internal"){
                        $("#storage-allocation").hide();
                    }
                    break;
                case 'NFS-Edit-Archive':
                    $scope.nfs = {};
                    $scope.type = "archive";
                    $scope.pattern = /^[a-zA-Z0-9_-]*$/;
                    StorageEdit = true;
                    nfsData = $scope.ngDialogData;
                    $scope.nfs.name = nfsData.name;
                    $("#title").text('Edit NFS Backup Copy Target');
                    $scope.nameErrorString = "Name should not have any space";
                    $("#id-name").disable();
                    $("#storage-allocation").hide();
                    $scope.nfs.host = nfsData.properties.hostname;
                    $scope.nfs.port = nfsData.properties.port;
                    $scope.nfs.share = nfsData.properties.share_name;
                    $scope.nfs.username = nfsData.properties.username;
                    $scope.nfs.password = nfsData.properties.password;
                    $scope.id = nfsData.id;
                    break;
            }
            processDialog(PHD, window);
        };

        function processDialog(PHD, window){
            var $port = $("#id_port"),
                $sizeValue = $("#size"),
                portSpinnerOptions = {"min": 1, "max": 10000, "step": 1},
                sizeSpinnerOptions = {"min": 1, "max": 1024, "step": 1};

            $scope.nfs.port = '2049';
            $scope.nfs.choice = "useActualValues";

            var $form = $("form[name=NfsForm]");
            $scope.$form = $form;

            var applianceForm = PHD.FormController($form, {
                serialize: function ($form) {
                    return $form.find("input").serialize();
                }
            });

            applianceForm
                .validate({
                    rules: {
                        name: {
                            required: true,
                            pattern: $scope.pattern
                        },
                        host: {
                            required: true
                        },
                        share: {
                            required: true
                        }
                    },
                    messages: {
                        name: {
                            required: gettext("Storage name is required"),
                            pattern:gettext($scope.nameErrorString)
                        },
                        host: {
                            required: gettext("Host is required")
                        },
                        share: {
                            required: gettext("Share is required")
                        }
                    }
                })


            $port.spinner(portSpinnerOptions);
            $sizeValue.spinner(sizeSpinnerOptions);
            $("#percent").disable();
            $("#percentValue").disable();
        }

        $scope.SizeChoiceClicked = function() {
            if($scope.nfs.choice == "usePercent"){
                $("#size").disable();
                $("#sizeMetric").disable();
                $("#percent").enable();
                $("#percentValue").enable();
                $("#size").spinner("disable");
            }
            else{
                $("#percent").disable();
                $("#percentValue").disable();
                $("#size").enable();
                $("#sizeMetric").enable();
                $("#size").spinner("enable");
            }
        };

        $scope.setDialogHeight = function(height){
                $(".ngdialog-content").height(height).css({
                    height: height + "px;"
                });
        };

        $scope.submitForm = function () {

            if ((!$scope.$form.valid())) {
                return false;
            }

            var obj = {};
            var properties = {};
            obj.type = 4;
            if(!StorageEdit){
                obj.name = $scope.nfs.name;
                obj.usage = $scope.type;
                properties.username = $scope.nfs.username;
                properties.password = $scope.nfs.password;
                if($scope.type === "stateless"){
                    obj.name = "_Stateless";
                    obj.build_filesystem = 1;
                }
            }
            else{
                obj.id = $scope.id;
                properties.username = $scope.nfs.username;
                properties.password = $scope.nfs.password;
                if(nfsData.type == "internal"){
                    obj.stateless_type = 4;
                    obj.type = 0;
                }
            }

            if($scope.type == "backup"){
                if(nfsData.stateless_type == null){
                    if($scope.nfs.choice == "usePercent"){
                        if(!$scope.NfsForm.percent.$pristine || $scope.nfs.percent == 100){
                            percent_size = $scope.nfs.percent;
                            var availableGB = $scope.nfs.mb_available/1024;
                            properties.max_size = ((percent_size*availableGB)/100);
                        }
                        else{
                            console.log($scope.nfs.percent);
                            properties.max_size=0;
                        }
                    }
                    else{
                        size_entered = $("#size").val();
                        if(size_entered == 0 || size_entered ===''){
                            properties.max_size = 0;
                        }
                        else{
                            if($scope.nfs.size == 2){
                                properties.max_size = parseInt(size_entered *1024);
                            }
                            else{
                                properties.max_size = parseInt(size_entered);
                            }
                        }
                    }
                }
            }


            properties.hostname = $scope.nfs.host;
            properties.port = $scope.nfs.port;
            properties.protocol = "nfs";
            properties.share_name = $scope.nfs.share;

            obj.properties = properties;

            var base_url = "/api/storage/";

            obj = JSON.stringify(obj);
            if(StorageEdit){
                console.log("PUT called");
                base_url +=$scope.id + "/";
                AddStorage = PHD.showLoadingIndicator("body", true, "Saving...");
                resp = PHD.Ajax.put(base_url + "?sid=" + PHD.appliance_sid, obj, null, handleStorageError);
            }
            else{
                if($scope.type == "archive") {
                    AddStorage = PHD.showLoadingIndicator("body", true, "Adding and checking for existing Backup Copies.  Please wait...");
                } else {
                    AddStorage = PHD.showLoadingIndicator("body", true, "Adding Storage...");
                }
                resp = PHD.Ajax.post(base_url + "?sid=" +  PHD.appliance_sid, obj, null, handleStorageError);
            }

            resp.done(function(data) {

                PHD.hideLoadingIndicator(AddStorage);

                if($scope.type == "archive") {
                    var result = data.storage;
                } else {
                    var result = data.result;
                    if (Array.isArray(result)) {
                        result = result[0];
                    }
                }

                if (result !== undefined) {
                    if(parseInt(result.code) === AJAX_RESULT_SUCCESS) {
                        console.log("SUCCESS");
                        processDialog(PHD, window);
                        $scope.closeThisDialog();
                    } else if(parseInt(result.code) === AJAX_RESULT_ERROR) {
                        console.log("FAILURE");
                        ngDialog.open({
                            dialogType: 'ERROR',
                            dialogMessage: data,
                            overlay:true
                        });
                    }
                    else{
                        if(!StorageEdit && $scope.type == "backup"){
                            ngDialog.open({
                                dialogType:"Information",
                                dialogMessage:"Storage added successfully! To finish the process, use the Storage Allocation box to allocate the amount of the NAS to be used for backup.",
                                modelDialogId:'ErrorDialog',
                                scope:$scope,
                                onConfirmOkButtonClick:'onConfirmError()'
                            });
                            $("#storage-allocation").show();
                            $scope.setDialogHeight(525);
                            StorageEdit = true;
                            $scope.id = result.id;
                            resp = PHD.Ajax.get(base_url + $scope.id + "/?sid=" +  PHD.appliance_sid, null, null, handleStorageError);
                            resp.done(function(data) {
                                $scope.nfs.mb_available = data.storage.properties.mb_available;
                            });
                        }
                        else if($scope.type == "archive"){
                            console.log("SUCCESS");
                            var dialogType = "Information";
                            if(result['messages'] != undefined && Array.isArray(result['messages'])) {
                                dialogType = "Warning";
                                var message = '';
                                for(var i=0; i<result['messages'].length;i++){
                                    message = message +"<br>"+result.messages[i];
                                }
                            } else if (result['sets_needing_import'] == 0) {
                                message = 'No backup copies found needing import';
                            } else {
                                message = 'All backup copies imported successfully';
                            }

                            ngDialog.open({
                                dialogType:dialogType,
                                dialogMessage:message,
                                modelDialogId:'InfoDialog',
                                scope:$scope,
                                onConfirmOkButtonClick:'onConfirmInfo()'
                            });
                        }
                    }
                }
                //PHD.currentDialog.wizard("close");
            });
        };

        function handleStorageError(jqXHR, textStatus, errorThrown) {
            PHD.hideLoadingIndicator(AddStorage);
            if(jqXHR.status === 500) {
                var data = jqXHR.responseJSON;
                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                    var error = data.result[0].message;
                    console.log("The error is:" + error);

                    if(error.indexOf("Storage configuration failed - this volume contains existing data, are you sure you want to format it?") > -1){               //Check if storage contains existing data, so we can show user option to recover.
                        var modifiedMessage = "The selected storage contains an existing filesystem.  If it contains Unitrends backup data, you can attempt to recover it.  Otherwise, you can erase the data or cancel and select different storage."
                        ngDialog.open({
                            dialogType:'CONFIRMATION - STORAGE',
                            modelDialogId:'ChooseRecoverOrErase',
                            scope:$scope,
                            dialogMessage: modifiedMessage,
                            tertiaryButton: true,
                            tertiaryButtonLabel: "Erase Data",
                            onConfirmOkButtonClick:'onRecover()',
                            onTertiaryButtonClick:'OnErase()'
                        });
                    }
                    else{
                        ngDialog.open({
                            dialogType:'Error',
                            modelDialogId:'ErrorDialog',
                            scope:$scope,
                            dialogMessage: error,
                            onConfirmOkButtonClick:'onConfirmError()'
                        });
                    }
                } else {
                    PHD.throwError(data);
                }
            } else {
                PHD.Ajax.ajaxError(jqXHR, textStatus, errorThrown);
            }
        }

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
        };

        $scope.onConfirmInfo = function () {
            ngDialog.close('InfoDialog');
            processDialog(PHD, window);
            $scope.closeThisDialog();
        };

        $scope.onRecover = function () {
            console.log("Recover the storage");
            $scope.build_filesystem = 0;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.OnErase = function () {
            console.log("Erase the Data");
            $scope.build_filesystem = 2;
            ngDialog.close('ChooseRecoverOrErase');
            $scope.submitForm();
        };

        $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });
    }]);

/*(function($, PHD, window, undefined) {
    
    var $form = $("#bds-form"),
        NFSForm,
        $port = $("#id_port"),
        $d2d = $("#id_d2d_allocation"),
        $ir = $("#id_ir_allocation"),
        NFS_REGEX = /^\\\\[a-zA-Z0-9\-\_\.]+\\.+$/,
        NAME_REGEX = /^[a-zA-Z0-9\-\_]+$/,
        $storageID = $("#edit_object_id"),
        $systemID = $("#id_system"),
        portSpinnerOptions = {"min": 1, "max": 10000, "step": 1},
        $toggleCtrl = $form.find(".formset-toggle :checkbox"),
        system_id = PHD.appliance_sid,
        allocationSpinnerOptions = {"min": 1, "max": 99, "step": 1};

    function initForm() {
        $form.find(".definition").popover({
            appendTo: $form,
            position: {
                my: "left bottom",
                at: "right top-15"
            }
        });

        $port.spinner(portSpinnerOptions);
        $d2d.spinner(allocationSpinnerOptions);
        $ir.spinner(allocationSpinnerOptions);

        NFSForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    name: {
                        required: true,
                        pattern: NAME_REGEX
                    },
                    host: {
                        required: true
                    },
                    share: {
                        required: true
                    }
                    /*
                    path: {
                        required: true,
                        pattern: NFS_REGEX
                    },
                    */
           /*     },
                messages: {
                    name: {
                        required: gettext("Please name your Storage")
                    },
                    host: {
                        required: gettext("Please provide the NFS hostname or IP address")
                    },
                    share: {
                        required: gettext("Please provide the NFS share name")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                $(document).trigger("bdssave", [system_id, data]);
                PHD.currentDialog.wizard("close");
            });
            /*
            .on("formsubmit", function(event, data) {
                    PHD.confirmDialog('<p>' + data.result[0].message + '</p>', {'title': gettext('Confirm Storage Edit')})
                        .done(function() {
                            $override.appendTo($form);
                            BDSForm.submit();
                        });
                }
            });
            */

      /*  if($toggleCtrl.length) {
            var $toManage = $("#id_password, #id_username").prop("disabled", true);

            $toggleCtrl.on("change", function(event) {
                $toManage.prop("disabled", !$(this).prop("checked"));
            });
        }
    }

    /*
    *   DOM Ready Function
    */
  /*  $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
        });

    });

})(jQuery, window.PHD || (window.PHD = {}), window); */
