var recoverModule = angular.module('recoveryModule', ['ngDialog', 'angulartics', 'angulartics.google.analytics','ui.grid','ui.grid.selection']);

recoverModule.config(['ngDialogProvider', function (ngDialogProvider) {
    ngDialogProvider.setDefaults({
        plain: false,
        closeByDocument: true,
        closeByEscape: true,
        appendTo: false,
        preCloseCallback: function (value) {

        }
    });
}]);

recoverModule.factory('recoverFactory', function($http,ngDialog) {
    return {
        getRestoreTargets: function(method, url, sid, backup_id, instance_id, client_id) {
            var params = {};
            if (sid != 0) {
                params.sid = (selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : sid;
                console.log("sab:  sid in getRestoreTargets: " + sid);
                if((app_type != undefined) && (app_type === APPLICATION_TYPE_NAME_HYPER_V)){
                    if(isIRJob) {
                        params.instance_id = instance_id;
                        params.type = 'ir';
                    } else {
                        params.backup_id = backup_id;
                        params.type = 'restore';
                    }
                } else {
                    params.backup_id = backup_id;
                    if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                        params.client_id = client_id;
                    }
                }
            }
            var promise = $http({
                url: url,
                method: method,
                params: params
            }).success(function(data, status, headers){

            }).error(function(response){
                ngDialog.open({
                    dialogType:'error',
                    dialogMessage:response.result[0].message
                });
            });
            return promise;
        }
    };
});

recoverModule.factory('applianceNetworkFactory', function($http,ngDialog) {
    return {
        getNetworkDetails: function(method, url, sid) {
            var params = {};
            if (sid != 0) {
                params.sid = sid;

            }
            var promise = $http({
                url: url,
                method: method,
                params: params
            }).success(function(data, status, headers){

            }).error(function(response){
                ngDialog.open({
                    dialogType:'error',
                    dialogMessage:response.result[0].message
                });

            });
            return promise;
        }
    };
});

recoverModule.factory('hypervisorSwitchFactory', function($http, ngDialog) {
   return {
       getHypervisorSwitch: function(method, url, hypervisor, sid) {
           var params = {};
           if (sid != 0) {
               params.sid = sid;
           }
           params.hypervisor_type = APPLICATION_TYPE_NAME_HYPER_V;
           params.hypervisor = hypervisor;
           var promise = $http({
               url: url,
               method: method,
               params: params
           }).success(function(data, status, headers){

           }).error(function(response){
               ngDialog.open({
                   dialogType:'error',
                   dialogMessage:response.result[0].message
               });

           });
           return promise;
       }
   };
});

recoverModule.factory('alternateExchangeServersFactory', function($http, ngDialog) {
    return {
        getAlternateExchangeServers: function(method, url, app_id, sid) {
            var params = {};
            if (sid != 0) {
                params.sid = sid;
            }
            params.app_id = app_id;

            var promise = $http({
                url: url,
                method: method,
                params: params
            }).success(function(data, status, headers){

            }).error(function(response){
                ngDialog.open({
                    dialogType:'error',
                    dialogMessage:response.result[0].message
                });

            });
            return promise;
        }
    };
});

recoverModule.factory('alternateLocationsFactory', function($http, ngDialog) {
    return {
        getAlternateLocations: function(method, url, sid) {
            var params = {};
            if (sid != 0) {
                params.sid = sid;
            }
            params.os_family = "Windows";
            params.showSystemClient = '1';

            var promise = $http({
                url: url,
                method: method,
                params: params
            }).success(function(data, status, headers){

            }).error(function(response){
                ngDialog.open({
                    dialogType:'error',
                    dialogMessage:response.result[0].message
                });

            });
            return promise;
        }
    };
});

recoverModule.controller('recoveryCtrl', ['$scope', '$rootScope', '$http', '$timeout', 'ngDialog', 'wizardService', '$analytics', 'recoverFactory', 'applianceNetworkFactory', 'hypervisorSwitchFactory', 'alternateExchangeServersFactory', 'alternateLocationsFactory',
    function($scope, $rootScope, $http, $timeout, ngDialog, wizardService, $analytics, recoverFactory, applianceNetworkFactory, hypervisorSwitchFactory, alternateExchangeServersFactory, alternateLocationsFactory) {

        $scope.jobName = "On-Demand";
        $scope.replicated = false;
        $scope.hideSwitch = true;
        $scope.hideRP = app_type !== APPLICATION_TYPE_NAME_VMWARE;
        $scope.hideStorage = false;
        $scope.isCluster = false;

        $scope.target = {};
        $scope.target.locationObj = "";
        $scope.recovery_asset = "";
        $scope.target.resourcePoolObj = "";
        $scope.target.dataStoreObj = "";
        $scope.target.appliance_network = "";
        $scope.target.network_switch = "";
        $scope.target.audit = false;

        $scope.locations = [];
        $scope.resourcePools = [];
        $scope.dataStores = [];
        $scope.applianceNetworks = [];
        $scope.networkSwitches = [];

        $scope.hostRequired = false;
        $scope.resourcePoolRequired = false;
        $scope.storageRequired = false;
        $scope.networkRequired = false;
        $scope.switchRequired = false;

        $scope.targetsData = [];
        $scope.locationName = "";
        $scope.hypervisorId = "";
        $scope.resourcePoolName = "";
        $scope.networkName = "";
        $scope.networkIP = "";

        $scope.restoreObj = {};
        $scope.targetStorageSelectPlaceHolder = false;
        $scope.hyperVTypeApplicationFlag = false;
        $scope.suffix = {};
        $scope.suffix.name = "_restore";

        //Xen variables
        $scope.notXen = true;
        $scope.storageRepos = [];
        $scope.storageName = "";
        $scope.target.storageObj = "";

        // Physical server variables
        $scope.physicalTarget = {};
        $scope.physicalTarget.asset = "";
        $scope.physicalServerRecoveryOption = {};
        $scope.physicalServerRecoveryOption.target = {
            flat: true,
            non_destructive: true,
            newer: false,
            today: false,
            unix: true,
            directory: ""
        };
        var defalutAssetVal = 1;

        // Oracle and SharePoint variables
        $scope.recoveryDataFirst = "";
        $scope.recoveryDataSecond = "";
        $scope.oracleSharePointObj = {};

        $scope.summaryHeaderArray = [];
        $scope.showSummaryHeader = true;

        //exchange server variables
        $scope.exchangeTarget = {};
        $scope.exchangeTarget.asset = "";
        $scope.exchangeTarget.recoveryTarget = "";
        $scope.exchangeTarget.recoveryArea = "";
        $scope.exchangeTarget.altAsset = "";
        $scope.exchangeTargetDataArray = [];
        $scope.assetTargets = [];
        $scope.altAssetTargets = [];
        $scope.exchangeRecoveryOption = {};
        $scope.exchangeRecoveryOption.target = {};
        $scope.exchangeOption = {};
        $scope.exchangeOption.target = {
            flat: true,
            non_destructive: true,
            newer: false,
            today: false,
            unix: true,
            directory: ""
        };
        $scope.recoveryAreas = [];
        $scope.recoveryDatabase = [];
        $scope.exchangeAssetName = "";
        $scope.isOriginalDB = false;
        $scope.exchangeRecoveryDatabasePlaceHolder = "";
        $scope.exchangeRecoveryDatabaseName = "";

        // SQL variables
        $scope.SQLRecoveryObj = {};
        $scope.SQLRecoveryObj.target = {};
        $scope.SQLRecoveryObj.target.instance = "";
        $scope.SQLRecoveryObj.target.database = "";
     //   $scope.SQLRecoveryObj.target.directory = "";
        $scope.SQLTarget = {};
        $scope.SQLTarget.asset = "";
        $scope.SQLTarget.directory = "";
        $scope.SQLTarget.recoveryTarget = "";
        $scope.SQLTargetDataArray = [];
        $scope.SQLselectedaltlocationFlag = false;
        $scope.selectedAssetNameChangeFlag = true;
        $scope.SQLTarget.recoveryOption = '1';
        $scope.SQLTarget.instanceObj = "";
        $scope.databaseRequired = false;
        $scope.instanceRequired = false;
        $scope.targetRequired = false;

        console.log(app_type + ":::" + selectedBackup.vm_name);

        if (app_type === APPLICATION_TYPE_NAME_SHAREPOINT || app_type === APPLICATION_TYPE_NAME_ORACLE) {
            $scope.summaryHeaderArray = [{
                "labelName": "Backup Details",
                "labelDescription": selectedBackup.backup_type + " " + selectedBackup.backup_date
            },
                {"labelName": "Size", "labelDescription": selectedBackup.backup_size + " MB"}
            ];
        } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
            $scope.summaryHeaderArray = [
                {
                    "labelName": "Backup",
                    "labelDescription": selectedBackup.backup_type + " " + selectedBackup.backup_date
                },
                {"labelName": "Database", "labelDescription": selectedBackup.vm_name},
                {"labelName": "Instance", "labelDescription": selectedBackup.server_instance_name},
                {"labelName": "Size", "labelDescription": selectedBackup.backup_size + " MB"}
            ];
        } else {
            $scope.summaryHeaderArray = [{"labelName": "Asset Name", "labelDescription": selectedBackup.vm_name},
                {
                    "labelName": "Backup Details",
                    "labelDescription": selectedBackup.backup_type + " " + selectedBackup.backup_date
                },
                {"labelName": "Size", "labelDescription": selectedBackup.backup_size + " MB"}
            ];
        }

        if (app_type == APPLICATION_TYPE_NAME_HYPER_V) {
            $scope.targetStorageSelectPlaceHolder = "Please select volume";
            $scope.notXen = true;
            if (isIRJob) {
                $scope.hyperVTypeApplicationFlag = false;
            } else {
                $scope.hyperVTypeApplicationFlag = true;
            }
        } else if (app_type == APPLICATION_TYPE_NAME_XEN) {
            $scope.notXen = false;
            $scope.hideStorage = true;
            $scope.hyperVTypeApplicationFlag = false;
        } else {
            $scope.targetStorageSelectPlaceHolder = "Please select datastore";
            $scope.hyperVTypeApplicationFlag = false;
            $scope.notXen = true;
        }

        if (app_type === APPLICATION_TYPE_NAME_SHAREPOINT) {
            $scope.recoveryDataFirst = "SharePoint farm will be recovered to its original location.";
            $scope.recoveryDataSecond = "To recover individual SharePoint items, use File Recovery options to create a file recovery object.";

        }
        if (app_type === APPLICATION_TYPE_NAME_ORACLE) {
            $scope.recoveryDataFirst = "The Oracle database will be recovered to its original location.";
            $scope.recoveryDataSecond = "To recover individual Oracle items, use File Recovery options to create a file recovery object.";
        }

        if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
            $scope.targetServerSelectPlaceHolder = "Please select a server";
            $scope.targetInstanceSelectPlaceHolder = "Please select an instance";

            $scope.recoveryTargetChange = function(asset){
                //get the sql instances
                $scope.instances = [];
                if (typeof($scope.locations) != "undefined") {
                    for (var i = 0; i < $scope.locations.length; i++) {
                        if ($scope.locations[i].value != "undefined") {
                            if($scope.locations[i].value == asset){
                                $scope.SQLTarget.assetName = $scope.locations[i].name;
                                for (var j = 0; j < $scope.locations[i].instances.length; j++) {
                                    $scope.instances.push({'name': $scope.locations[i].instances[j].instance_name});
                                }
                            }
                        }
                    }
                }
                $scope.SQLselectedaltlocationFlag = parseInt(selectedBackup.client_id,10) !== asset;
            }
        }

        if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
            var originalServer = {};
            originalServer.value = 1;
            originalServer.name = "Original Exchange Server";
            $scope.exchangeTargetDataArray.push(originalServer);

            if (selectedBackup.app_id != 2) {
                var altExchangeServer = {};
                altExchangeServer.value = 2;
                altExchangeServer.name = "Alternate Exchange Server";
                $scope.exchangeTargetDataArray.push(altExchangeServer);
            }

            var altLocation = {};
            altLocation.value = 3;
            altLocation.name = "Alternate Location";
            $scope.exchangeTargetDataArray.push(altLocation);
        }

        /********* Exchange Recovery Target change *********/
        $scope.exchangeRecoveryTargetChange = function () {
            $scope.recoveryAreas = [];
            $scope.recoveryDatabase = [];
            $scope.assetTargets = [];
            $scope.altAssetTargets = [];
            $scope.exchangeTarget.asset = "";
            $scope.exchangeTarget.altAsset = "";
            $scope.exchangeTarget.recoveryArea = "";
            $scope.exchangeAssetName = "";

            if ($scope.exchangeTarget.recoveryTarget === 1) {
                $scope.getRecoveryDatabases(selectedBackup.client_id);
                $scope.exchangeAssetName = selectedBackup.client_name;
            } else if (selectedBackup.app_id != 2 && $scope.exchangeTarget.recoveryTarget === 2) {
                
                var url = "/api/clients/filtered/",
                    load = PHD.showLoadingIndicator("body", true, "Loading..."),
                    app_id = parseInt(selectedBackup.app_id, 10);
                alternateExchangeServersFactory.getAlternateExchangeServers('get', url, app_id, selectedBackup.sid)
                    .then(function (data) {
                        var applicable_servers = data.data.data;
                        if (applicable_servers && applicable_servers.length > 0) {
                            angular.forEach(applicable_servers, function (value, key) {
                                var assetTempObj = {};
                                // passing all the clients except the original client
                                if (value.client_id != selectedBackup.client_id) {
                                    assetTempObj.value = value.client_id;
                                    assetTempObj.name = value.client_name;
                                    $scope.assetTargets.push(assetTempObj);
                                }
                            });
                        }
                        PHD.hideLoadingIndicator(load);
                    });
                
            } else {
                var url = "/api/clients/filtered/",
                    load = PHD.showLoadingIndicator("body", true, "Loading...");
                alternateLocationsFactory.getAlternateLocations('get', url, selectedBackup.sid)
                    .then(function (data) {
                        var applicable_servers = data.data.data;
                        if (applicable_servers && applicable_servers.length > 0) {
                            angular.forEach(applicable_servers, function (value, key) {
                                var assetTempObj = {};
                                assetTempObj.value = value.client_id;
                                assetTempObj.name = value.client_name;
                                $scope.altAssetTargets.push(assetTempObj);
                            });
                        }
                        PHD.hideLoadingIndicator(load);
                    });
            }
        };

        $scope.exchangeAssetChange = function () {
            if ($scope.exchangeTarget.asset !== null) {
                $scope.getRecoveryDatabases($scope.exchangeTarget.asset);
                $scope.exchangeAssetName = $.grep($scope.assetTargets, function (asset) {
                    return asset.value == $scope.exchangeTarget.asset;
                })[0].name;
            } else {
                $scope.exchangeAssetName = "";
            }
        };

        $scope.exchangeAltAssetChange = function() {
            $scope.exchangeAssetName = $.grep($scope.altAssetTargets, function (asset) {
                return asset.value == $scope.exchangeTarget.altAsset;
            })[0].name;
        };

        $scope.exchangeRecoveryAreaChange = function () {
            $scope.exchangeRecoveryDatabasePlaceHolder = "Please select recovery database";
            $scope.recoveryAreas = $scope.recoveryDatabase;
        };

        $scope.exchangeOriginalLocationChange = function () {
            $scope.exchangeAssetName = selectedBackup.client_name;
        };

        $scope.getRecoveryDatabases = function (client_id) {
            $scope.recoveryDatabase = [];
            $scope.recoveryAreas = [];
            $scope.isOriginalDB = false;
            $scope.exchangeTarget.recoveryOption = 2;
            var url = "/api/restore/targets/",
                load = PHD.showLoadingIndicator("body", true, "Loading...");
            recoverFactory.getRestoreTargets('get', url, ((selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : selectedBackup.sid), selectedBackup.backup_id, selectedBackup.instance_id, client_id)
                .then(function (data) {
                    PHD.hideLoadingIndicator(load);
                    var restore_targets = data.data.targets;
                    if (restore_targets && restore_targets.length > 0) {
                        angular.forEach(restore_targets, function (value, key) {
                            if (selectedBackup.vm_name === value.name) {
                                $scope.isOriginalDB = true;
                                $scope.exchangeTarget.recoveryOption = 1;
                            } else {
                                $scope.recoveryDatabase.push(value.name);
                            }
                        });
                    } 
                    if ($scope.exchangeTarget.recoveryOption == 2 && $scope.recoveryDatabase.length > 0) {
                        $scope.recoveryAreas = $scope.recoveryDatabase;
                    }
                });
        };

        //API call for step first "Recovery Options".
        var url = "/api/restore/targets/",
            load = PHD.showLoadingIndicator("body", true, "Loading Target Location...");
        recoverFactory.getRestoreTargets('get', url, ((selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : selectedBackup.sid), selectedBackup.backup_id, selectedBackup.instance_id, selectedBackup.client_id)
            .then(function (data) {
                PHD.hideLoadingIndicator(load);
                var restore_targets = data.data.targets;
                if (restore_targets && restore_targets.length > 0) {
                    if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                        angular.forEach(restore_targets, function (value, key) {
                            var locationTempObj = {};
                            locationTempObj.value = value.client_id;
                            locationTempObj.name = value.client_name;
                            if (value.client_name == selectedBackup.vm_name) {
                                defalutAssetVal = value.client_id;
                            }
                            $scope.locations.push(locationTempObj);
                        });
                    } else {
                        $scope.isOriginalDB = false;
                        $scope.exchangeTarget.recoveryOption = 2;
                        angular.forEach(restore_targets, function (value, key) {
                            var locationTempObj = {};
                            if (app_type === APPLICATION_TYPE_NAME_SHAREPOINT || app_type === APPLICATION_TYPE_NAME_ORACLE) {
                                if (app_type === APPLICATION_TYPE_NAME_SHAREPOINT && value.num_servers > 1) {
                                    $scope.recoveryDataFirst = "This is a multi-server Farm. Full restores are only supported if SharePoint is on single server.";
                                    $scope.recoveryDataSecond = "To restore a multi-server Farm, use File Level Recovery.";
                                    $scope.isMultiServer = true;
                                } else {
                                    $scope.oracleSharePointObj.client_id = value.client_id;
                                    $scope.isMultiServer = false;
                                }
                            } else if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                                $scope.exchangeAssetName = selectedBackup.client_name;
                                if (selectedBackup.vm_name === value.name) {
                                    $scope.isOriginalDB = true;
                                    $scope.exchangeTarget.recoveryOption = 1;
                                } else {
                                    $scope.recoveryDatabase.push(value.name);
                                }
                                if (!$scope.isOriginalDB) {
                                    $scope.exchangeRecoveryDatabasePlaceHolder = "Please select recovery database";
                                    $scope.recoveryAreas = $scope.recoveryDatabase;
                                }
                            } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
                                locationTempObj.value = value.client_id;
                                locationTempObj.name = value.client_name;
                                locationTempObj.database = value.database;
                                locationTempObj.instances = value.instances;

                                $scope.locations.push(locationTempObj);
                                $scope.SQLTarget.database = selectedBackup.datbase_name;
                            } else {
                                locationTempObj.value = value.id;
                                locationTempObj.name = value.name;
                                $scope.locations.push(locationTempObj);
                            }
                        });
                    }
                } else if (angular.isObject(restore_targets)){
                    var locationTempObj = {};
                    $scope.isSystemDB = false;

                    if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
                        locationTempObj.value = restore_targets.client_id;
                        locationTempObj.name = restore_targets.client_name;
                        locationTempObj.database = restore_targets.database;
                        locationTempObj.instances = restore_targets.instances;

                        $scope.locations.push(locationTempObj);
                        $scope.SQLTarget.database = selectedBackup.datbase_name;
                        $scope.SQLTarget.isSystemDB = restore_targets.system_db;
                    }
                }
                $scope.targetsData = restore_targets;
                if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                    $scope.physicalTarget.asset = defalutAssetVal;
                    $scope.targetNameForPhysicalServer = $.grep($scope.locations, function (location) {
                        return location.value == $scope.physicalTarget.asset;
                    })[0].name;
                }
            });

        if (isIRJob) {
            $scope.title = "Create Instant Recovery Job";
            $scope.notIR = false;
            $scope.hideSwitch = app_type !== APPLICATION_TYPE_NAME_HYPER_V;

            var network_url = "/api/networks/";
            applianceNetworkFactory.getNetworkDetails('get', network_url, (selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : selectedBackup.sid)
                .then(function (data) {
                    var restore_networks = data.data.networks;
                    if (restore_networks && restore_networks != undefined && restore_networks.length > 0) {
                        angular.forEach(restore_networks, function (network, index) {
                            var networkTempObj = {};
                            networkTempObj.value = network.ip;
                            networkTempObj.name = network.id + " - " + network.ip;
                            $scope.applianceNetworks.push(networkTempObj);
                        });
                    }
                });
            $scope.networkRequired = true;

            //Hide/Unhide the network switches
            if (app_type === APPLICATION_TYPE_NAME_HYPER_V) {
                $scope.hideSwitch = $scope.target.audit;
                if (!$scope.target.audit) {
                    $scope.switchRequired = true;
                } else {
                    $scope.switchRequired = false;
                }
            }
        } else {
            $scope.title = "Create Recover Job";
            $scope.notIR = true;
            $scope.hideSwitch = true;
            $scope.networkRequired = false;
        }

        $scope.locationChange = function (location) {
            $scope.resourcePools = [];
            if ($scope.target.locationObj !== "" && typeof($scope.target.locationObj) !== "undefined") {
                $scope.resourcePools = [];
                for (var i = 0; i < $scope.targetsData.length; i++) {
                    if (typeof($scope.targetsData[i].groups) !== 'undefined') {
                        if ($scope.targetsData[i].id == location) {
                            $scope.target.resourcePoolObj = "";
                            angular.forEach($scope.targetsData[i].groups.VG, function (value, key) {
                                var obj = {
                                    name: value.Name,
                                    value: value.Key
                                };
                                $scope.resourcePools.push(obj);
                            });
                        }
                    }
                }
                $scope.dataStores = [];
                $scope.storageRepos = [];
                if (typeof($scope.targetsData) != "undefined") {
                    for (var i = 0; i < $scope.targetsData.length; i++) {
                        if ($scope.targetsData[i].id == location) {
                            if (typeof($scope.targetsData[i].datastores) != "undefined") {
                                if (app_type === APPLICATION_TYPE_NAME_XEN) {
                                    angular.forEach($scope.targetsData[i].datastores, function (value, key) {
                                        var obj = {
                                            name: value.name,
                                            value: value.id
                                        };
                                        $scope.storageRepos.push(obj);
                                    });
                                } else {
                                    for (var j = 0; j < $scope.targetsData[i].datastores.length; j++) {
                                        $scope.dataStores.push($scope.targetsData[i].datastores[j].name);
                                    }
                                }
                            }
                            if (typeof ($scope.targetsData[i].capabilities) != "undefined") {
                                if (typeof ($scope.targetsData[i].capabilities.is_cluster) != "undefined") {
                                    $scope.isCluster = $scope.targetsData[i].capabilities.is_cluster;
                                }
                            }
                        }
                    }
                }
                if (!isIRJob && app_type === APPLICATION_TYPE_NAME_HYPER_V && $scope.isCluster) {
                    $scope.hideStorage = true;
                    $scope.storageRequired = false;
                    $scope.target.dataStoreObj = "";
                } else {
                    if (app_type === APPLICATION_TYPE_NAME_XEN) {
                        $scope.hideStorage = true;
                        $scope.notXen = false;
                    } else {
                        $scope.hideStorage = false;
                        $scope.notXen = true;
                    }
                    $scope.storageRequired = true;
                }
            }
            console.log($scope.locations);
            if (app_type !== APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                $scope.locationName = $.grep($scope.locations, function (location) {
                    return location.value == $scope.target.locationObj;
                })[0].name;
            } else {
                $scope.targetNameForPhysicalServer = $.grep($scope.locations, function (location) {
                    return location.value == $scope.physicalTarget.asset;
                })[0].name;
            }

            // If Hyper-V IR, need to get network switches.
            if (isIRJob && app_type === APPLICATION_TYPE_NAME_HYPER_V) {
                $scope.networkSwitches = [];
                $scope.hypervisorId = $scope.locationName;
                var switch_url = "/api/networks/hypervisor-switch/";
                var networkSwitchDiv = $("#id_switch_control_group");
                var networkSwtichDivId = $("#id_switch");
                load = PHD.showLoadingIndicator(networkSwtichDivId);
                hypervisorSwitchFactory.getHypervisorSwitch('get', switch_url, $scope.hypervisorId, (selectedBackup.replicated == 'true') ? selectedBackup.local_system_id : selectedBackup.sid)
                    .then(function (data) {
                        PHD.hideLoadingIndicator(load);
                        var restore_switches = data.data.names;
                        if (restore_switches) {
                            angular.forEach(restore_switches, function (name, index) {
                                $scope.networkSwitches.push(name);
                            })
                        }
                    });
            }
        };

        $scope.resourcePoolChange = function (resourcePool) {
            $scope.resourcePoolName = $.grep($scope.resourcePools, function (resourcePool) {
                return resourcePool.value == $scope.target.resourcePoolObj;
            })[0].name;
        };

        $scope.storageRepoChange = function (storage) {
            $scope.storageName = $.grep($scope.storageRepos, function (storage) {
                return storage.value == $scope.target.storageObj;
            })[0].name;
        };

        $scope.applianceNetworkChange = function (network) {
            $scope.networkName = $.grep($scope.applianceNetworks, function (network) {
                return network.value == $scope.target.appliance_network;
            })[0].name;
        };

        $scope.auditChange = function () {
            if (app_type === APPLICATION_TYPE_NAME_HYPER_V) {
                $scope.hideSwitch = $scope.target.audit;
            } else {
                $scope.hideSwitch = true;
            }
        };

        /********************************** Recover summary grid**************************/
        var editSuffixesCellTempate = "";

        if (app_type === APPLICATION_TYPE_NAME_VMWARE || isIRJob || app_type === APPLICATION_TYPE_NAME_XEN) {
            editSuffixesCellTempate = '<div style="padding-top:5px;" ng-click="grid.appScope.openEditSuffixesdialog(row.entity,row.getProperty(col.field), col.field)"><div style="font-size: 1.2em;text-overflow:ellipsis;float:left;">{{row.entity.vm_name}}</div><div style="float:right;padding-right:5px;" title="Edit Suffix"><img  src="assets/images/edit_pencil.png" style="height:14px;cursor:pointer;"/></div></div>';
        }
        var storageDisplayName = "";
        var assetDisplayName = "";
        var targetDisplayName = "";
        var instanceDisplayName = "";
        if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
            storageDisplayName = "TARGET DIRECTORY";
            assetDisplayName = "RECOVERY ASSET";
            targetDisplayName = "TARGET ASSET";
        } else if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
            storageDisplayName = "TARGET STORAGE";
            assetDisplayName = "RECOVERY ASSET";
            targetDisplayName = "TARGET ASSET";
        } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER){
            instanceDisplayName = "INSTANCE NAME";
            assetDisplayName = "DATABASE NAME";
            targetDisplayName = "TARGET HOST";
            storageDisplayName = "TARGET DIRECTORY";
        } else {
            storageDisplayName = "TARGET STORAGE";
            assetDisplayName = "VM NAME";
            targetDisplayName = "TARGET HOST";
        }

	    $scope.recoverGridOptions = {
		    enableColumnMenus:false,
			enableHorizontalScrollbar: 0,
            columnDefs: [
				{ field: 'vm_name',displayName:assetDisplayName,resizable:true,cellTemplate:editSuffixesCellTempate},
			    { field: 'target_host',displayName:targetDisplayName},
                { field: 'target_resource',displayName:"TARGET RESOURCE", visible:(app_type === APPLICATION_TYPE_NAME_VMWARE)? true:false },
                { field: 'instance_name', displayName:instanceDisplayName, visible:(app_type === APPLICATION_TYPE_NAME_SQL_SERVER) ? true : false},
                { field: 'target_storage',displayName:storageDisplayName},
                { field: 'appliance_network',displayName:"APPLIANCE NETWORK",visible:(isIRJob)? true:false },
				{ field: 'network_switch',displayName:"NETWORK SWITCH",visible:(isIRJob && app_type === APPLICATION_TYPE_NAME_HYPER_V)? true:false }
            ]
        };

		/********************************** Function to edit suffixes*********************/			
		$scope.openEditSuffixesdialog = function(row, cell, col) {
			$scope.selectedRow = row;
      		$scope.selectedColumn = col;
			$scope.VmName = row.vm_name;
            ngDialog.open({
                template: 'app/recover/editSuffixes.html',
                scope: $scope,
                overlay: true,
                modelDialogId: 'recover_edit_suffix_dialogId',
                ngDialogStyle: 'width:42em; height:26em;'
            });
            $scope.suffix.name = "_restore";
		};

		/********************************** Function to update suffixes*********************/			
		$scope.updateSuffixestToVMName = function(){ // Need to fix
			var suffix = $scope.suffix.name;
			var editAction =  $("#id_suffix_action option:selected").text();
		
			if(editAction == "Append") {
				$scope.getSummaryDetail($scope.VmName+suffix);
				ngDialog.close('recover_edit_suffix_dialogId');
			} else {
				 var vmName = $scope.VmName;
				 var updatedVmName = vmName.replace(suffix,"");
				 console.log(updatedVmName);
				 $scope.getSummaryDetail(updatedVmName);
				 ngDialog.close('recover_edit_suffix_dialogId');
			}
		};

		/**********************************Wizard next steps.*********************/
		$scope.nextStepWizard = function($event) {
			if (wizardTitle == "Define Recovery Options") {//1st step
                $scope.hostRequired = true;
                $scope.storageRequired = true;

                if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                    $scope.getSummaryDetail("");
                    $scope.showSummaryHeader = false;
                    $rootScope.$broadcast("nextBtnClick");
                } else {
					if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                        if (checkValidationForExchangeRecoverOptions()) {
                            $scope.showSummaryHeader = false;
                            $scope.getSummaryDetail("");
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
                        if (checkValidationForSqlRecoverOptions($scope.SQLselectedaltlocationFlag)) {
                            $scope.showSummaryHeader = false;
                            $scope.getSummaryDetail("");
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    } else {
                        if (checkValidationForRecoverOptions($scope.hostRequired, $scope.resourcePoolRequired, $scope.storageRequired, $scope.networkRequired, $scope.switchRequired)) {
                            $scope.showSummaryHeader = false;
                            $scope.getSummaryDetail("");
                            $rootScope.$broadcast("nextBtnClick");
                        }
                    }
                }
			}
			if (wizardTitle == "Summary") { //2nd step
				 $scope.showSummaryHeader = true;
				 $scope.postRecoverData(); //call to the POST API
            }
		};
      
		$scope.clickPreviousStep = function(){
			     $scope.showSummaryHeader = true;
		};

		/********************************** Function to get summary detail*********************/
		$scope.getSummaryDetail = function(vmNameParam){
            var network = "";
			var vmName = "";
            var switchName = "";
            var hostName = "";
            var storageName = "";
            var instanceName = "";
			if (vmNameParam != "" && vmNameParam != undefined) {
			    vmName = vmNameParam;
			} else if ((app_type == APPLICATION_TYPE_NAME_HYPER_V && !isIRJob) || app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER ||
                        app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                vmName = selectedBackup.vm_name;
            } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER){
                vmName = $scope.SQLTarget.database;
                storageName = $scope.SQLTarget.directory;
			} else {
				vmName = selectedBackup.vm_name+ '_restore'; //VM name should have '_restore' appended in case of vmware.
			}
            if (isIRJob) {
                network = $scope.networkName;
            }
            if (isIRJob && app_type === APPLICATION_TYPE_NAME_HYPER_V && !$scope.hideSwitch) {
                switchName = $scope.target.network_switch;
            }
            if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                hostName = $scope.targetNameForPhysicalServer;
                storageName = $scope.physicalServerRecoveryOption.target.directory;
            } else if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                hostName = $scope.exchangeAssetName;
                if ($scope.exchangeTarget.recoveryOption == 1) {
                    storageName = selectedBackup.vm_name;
                }
                if ($scope.exchangeTarget.recoveryOption == 2 && $scope.exchangeTarget.recoveryArea) {
                    storageName = $scope.exchangeTarget.recoveryArea;
                }
                if ($scope.exchangeTarget.recoveryTarget === 3) {
                    storageName = $scope.exchangeOption.target.directory;
                }
            } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER){
                hostName = $scope.SQLTarget.assetName;
                instanceName = $scope.SQLTarget.instance.name;
                storageName = $scope.SQLTarget.directory;
            } else {
                hostName = $scope.locationName;
                if (app_type === APPLICATION_TYPE_NAME_XEN) {
                    storageName = $scope.storageName;
                } else {
                    storageName = $scope.target.dataStoreObj;
                }
            }
			
			console.log($scope.targetNameForPhysicalServer);
            $scope.recoverGridOptions.data = [
                {
                    "vm_name": vmName,
                    "target_host": hostName,
                    "target_resource": $scope.resourcePoolName,
                    "target_storage": storageName,
                    "appliance_network": network,
					"network_switch":switchName,
                    "instance_name":instanceName
                }
            ];
            console.log($scope.recoverGridOptions.data);
		};
		
		/************************ Function to post the recover data to backend **********/
		$scope.postRecoverData = function() {
			
			$scope.restoreObj = {};
            $scope.restoreObj.target = {};

			if (app_type === APPLICATION_TYPE_NAME_VMWARE || app_type === APPLICATION_TYPE_NAME_HYPER_V || app_type === APPLICATION_TYPE_NAME_XEN) {
		       $scope.restoreObj  = $scope.collectRecoverData();
			} else if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER){
			   $scope.restoreObj  = $scope.collectRecoverDataForPhysicalServer();
			} else if (app_type === APPLICATION_TYPE_NAME_SHAREPOINT || app_type === APPLICATION_TYPE_NAME_ORACLE) {
                $scope.restoreObj = $scope.collectRecoverDataForOracleSharePoint();
            } else if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                $scope.restoreObj = $scope.collectRecoverDataForExchange();
            } else if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
                $scope.restoreObj = $scope.collectRecoverDataForSQL();
            }
			
            console.log('=== Submit form: object = ' + JSON.stringify($scope.restoreObj));
            var url = (isIRJob ? "/api/restore/instant/" : "/api/restore/full/");
            var params = {};
            params.sid = selectedBackup.replicated ? selectedBackup.local_system_id : selectedBackup.sid;
            var load = PHD.showLoadingIndicator("body", true, "Starting Recovery...");

            $http ({
                method: 'post',
                url : url,
                data: JSON.stringify($scope.restoreObj),
                params: params
            }).success(function(data){
                 PHD.hideLoadingIndicator(load);
                 ngDialog.open({
                    dialogType:'Information',
				    scope:$scope,
                    dialogMessage:(!isIRJob)?"Restore has started successfully" : "Instant Recovery has started successfully"
                });
				  //ngDialog.close('recover-dialog');
				
            }).error(function(response) {
                PHD.hideLoadingIndicator(load);
                ngDialog.open({
                    dialogType:'retry',
                    dialogMessage:response.result[0].message
                })
            });
		};
		
		/****************** Function to collect restore data of VMware and Hyper-V**************/
        $scope.collectRecoverData = function(){
	        if (app_type === APPLICATION_TYPE_NAME_HYPER_V) {
                if (isIRJob) {
                    $scope.restoreObj.target.host = $scope.locationName;
                } else {
                    $scope.restoreObj.client_id = parseInt($scope.target.locationObj, 10);
                }
            } else {
                $scope.restoreObj.target.host = $scope.target.locationObj;
            }

            $scope.restoreObj.target.name = $scope.recoverGridOptions.data[0].vm_name;
            console.log("VM name " + $scope.restoreObj.target.name);

            if (app_type === APPLICATION_TYPE_NAME_HYPER_V) {
                $scope.restoreObj.target.directory =  $scope.target.dataStoreObj;
            } else if (app_type === APPLICATION_TYPE_NAME_XEN) {
                $scope.restoreObj.target.storage_repository = $scope.target.storageObj;
                $scope.restoreObj.target.metadata = true;
            } else {
                $scope.restoreObj.target.datastore =  $scope.target.dataStoreObj;
                $scope.restoreObj.target.group = $scope.resourcePoolName;
                $scope.restoreObj.target.metadata = true;
                console.log("RP name " + $scope.restoreObj.target.group);
                if (!isIRJob) {
                    $scope.restoreObj.target.metadata = true;
                    $scope.restoreObj.target.template = selectedBackup.vmware_template;
                }
            }

            $scope.restoreObj.backup_id = parseInt(selectedBackup.backup_id, 10);
            if (isIRJob) {
                // start in audit mode.
                $scope.restoreObj.audit = $scope.target.audit;
                if (!$scope.restoreObj.audit && (app_type === APPLICATION_TYPE_NAME_HYPER_V)) {
                    $scope.restoreObj.target.switch = $scope.target.network_switch;
                }
                $scope.restoreObj.address = $scope.target.appliance_network;
            }
			return $scope.restoreObj;
		};

		/*************************** Function to collect physical server data****************************/
		$scope.collectRecoverDataForPhysicalServer = function() {
            $scope.physicalServerRecoveryOption.backup_id = parseInt(selectedBackup.backup_id, 10);
            $scope.physicalServerRecoveryOption.client_id = $scope.physicalTarget.asset;
            $scope.physicalServerRecoveryOption.synthesis = selectedBackup.synth_capable;

            $scope.physicalServerRecoveryOption.excludes = "";
            $scope.physicalServerRecoveryOption.includes = "";

            console.log($scope.physicalServerRecoveryOption);
            return $scope.physicalServerRecoveryOption;
	 	};

        /*************************** Function to collect Oracle and SharePoint data***********************/
        $scope.collectRecoverDataForOracleSharePoint = function() {
            $scope.oracleSharePointObj.backup_id = parseInt(selectedBackup.backup_id, 10);

            return $scope.oracleSharePointObj;
        };

        /***************************** Function to collect Exchange data*****************/
        $scope.collectRecoverDataForExchange = function() {
            $scope.exchangeRecoveryOption.backup_id = parseInt(selectedBackup.backup_id, 10);
            var client_id = selectedBackup.client_id;
            var database = "";
            if (!$scope.exchangeOriginalDisabled) {
                database = selectedBackup.vm_name;
            }
            if ($scope.exchangeTarget.recoveryOption == 2 && $scope.exchangeTarget.recoveryArea) {
                database = $scope.exchangeTarget.recoveryArea;
            }
            if ($scope.exchangeTarget.recoveryTarget === 2) {
                client_id = $scope.exchangeTarget.asset;
                if (database !== "") {
                    $scope.exchangeRecoveryOption.target.database = database;
                }
            } else {
                if ($scope.exchangeTarget.recoveryTarget === 3) {
                    client_id = $scope.exchangeTarget.altAsset;
                    $scope.exchangeRecoveryOption.target = $scope.exchangeOption.target;
                } else {
                    if (database !== "") {
                        $scope.exchangeRecoveryOption.target.database = database;
                    }
                }
            }
            $scope.exchangeRecoveryOption.client_id = client_id;
            return $scope.exchangeRecoveryOption;
        };

        /*************************** Function to collect SQL data***********************/
        $scope.collectRecoverDataForSQL = function() {
            $scope.SQLRecoveryObj.backup_id = (selectedBackup.backup_id);
            $scope.SQLRecoveryObj.target.instance =  $scope.SQLTarget.instance.name;
            $scope.SQLRecoveryObj.target.database =  $scope.SQLTarget.database;
            $scope.SQLRecoveryObj.client_id = $scope.SQLTarget.asset;
            if($scope.SQLTarget.directory !== "") {
                $scope.SQLRecoveryObj.target.directory = $scope.SQLTarget.directory;
            }

            console.log("sab:  SQL obj for POST: " + $scope.SQLRecoveryObj);

            return $scope.SQLRecoveryObj;
        };

        /************ Popover function for IR wizard*************/
	    $scope.showPopoverForIR = function(){
            var form1 = angular.element("#recovery-job-form");
		    console.log(form1);
		    form1.find(".definition").popover({
		        appendTo: form1
		    });
        };

        /************* Get Recovery Options based on app_type *******/
        $scope.getRecoveryOptions = function() {
            var include = '';
		    if (app_type === APPLICATION_TYPE_NAME_HYPER_V || app_type === APPLICATION_TYPE_NAME_VMWARE || app_type === APPLICATION_TYPE_NAME_XEN) {
			    include = 'app/recover/recoveryOptions.html';
            }
            if (app_type === APPLICATION_TYPE_NAME_PHYSICAL_SERVER) {
                include = 'app/recover/recoveryOptions/physicalServerOptions.html';
            }
            if (app_type === APPLICATION_TYPE_NAME_EXCHANGE) {
                include = 'app/recover/recoveryOptions/exchangeOptions.html';
                var exchange_form = angular.element("#exchange-recovery-job-form");
                exchange_form.find(".definition").popover({
                    appendTo: exchange_form
                });
            }
            if (app_type === APPLICATION_TYPE_NAME_SQL_SERVER) {
                include = 'app/recover/recoveryOptions/sqlOptions.html';
                var form1 = angular.element("#sql-recovery-job-form");
                console.log(form1);
                form1.find(".definition").popover({
                    appendTo: form1
                });
            }
            return include;
        };

        /*************** Get recovery Steps **********************/
        $scope.getRecoverySteps = function() {
            var recoverySteps = '';
            if (app_type === APPLICATION_TYPE_NAME_ORACLE || app_type === APPLICATION_TYPE_NAME_SHAREPOINT) {
                recoverySteps = 'app/recover/oracleSharePointRecovery.html';
            } else {
                recoverySteps = 'app/recover/recoverySteps.html';
            }
            return recoverySteps;
        };
		
        /*************** For add exclude patern**************************/
	    $scope.addExcludePatternRow = function() {
		    console.log($scope.physicalTarget.exclusion);
		    var table = document.getElementById("exclusionTable");
		    var excludePattern = $scope.physicalTarget.exclusion;

		    if(excludePattern == ""){
			   return;
			}
 		    var rowCount = table.rows.length;
		    var row = table.insertRow(rowCount);
 		    var cell1 = row.insertCell(0);
            cell1.innerHTML= excludePattern;
		
            var cell2 = row.insertCell(1);
            cell2.innerHTML =  '<span class="btn-remove-recipient"><a ng-show=false><i title="Remove" class="icon-removesign"></i></a></span>';
            $scope.physicalTarget.exclusion="";
        };

		/********************For remove exclude patern**************************/
		$scope.deleteRow = function(r) {
			console.log(r);
			var i = r.parentNode.parentNode.parentNode.rowIndex;
			document.getElementById("exclusionTable").deleteRow(i);
		};

        /********************* For SharePoint and Oracle submit****************/
        $scope.doRestore = function() {
            $scope.postRecoverData();
        }
    }

]);